@@ -4,7 +4,7 @@
 ====================================
  :mod:`ipcalc` IP subnet calculator
 ====================================
-.. moduleauthor:: Wijnand Modderman <http://tehmaze.com/>
+.. moduleauthor:: Wijnand Modderman-Lenstra <maze@pyth0n.org>
 .. note:: BSD License
 
 About
@@ -12,26 +12,6 @@ About
 
 This module allows you to perform network calculations.
 
-Changelog
-=========
-
-    ========== ===============================================================
-    Date       Changes
-    ========== ===============================================================
-    2009-04-11 Added docstrings.
-    2009-03-23 Added IPv4 short-hand form support, thanks to VeXocide.
-    2007-10-26 Added IPv6 support, as well as a lot of other functions, 
-               refactored the calculations.
-    2007-10-25 Initial writeup, because I could not find any other workable
-               implementation.
-    ========== ===============================================================
-
-Todo
-====
-
-Todo:
- * add CLI parser
-
 References
 ==========
 
@@ -48,19 +28,43 @@ Thanks
 ======
 
 I wish to thank the following people for their input:
- * Bastiaan (trbs)
+ * Bas van Oostveen (trbs)
  * Peter van Dijk (Habbie)
  * Hans van Kranenburg (Knorrie)
  * Jeroen Habraken (VeXocide)
- * Torbjörn Lönnemark
+ * Torbjörn Lönnemark (tobbez)
+ * Anthony Cornehl (twinshadow)
 
 '''
 
-__version__ = '0.5.1'
+__version__ = '0.6'
 
 import socket
 
 
+try:
+    bin(42)
+except NameError:
+    def bin(x):
+        '''
+        Stringifies an int or long in base 2.
+        '''
+        if x < 0: 
+            return '-' + bin(-x)
+        out = []
+        if x == 0: 
+            out.append('0')
+        while x > 0:
+            out.append('01'[x & 1])
+            x >>= 1
+            pass
+        try: 
+            return '0b' + ''.join(reversed(out))
+        except NameError, ne2: 
+            out.reverse()
+        return '0b' + ''.join(out) 
+
+
 class IP(object):
     '''
     Represents a single IP address.
@@ -151,7 +155,7 @@ class IP(object):
                 self.v = version or 4
                 self.dq = self._itodq(ip)
             else:
-                self.v = version or 4
+                self.v = version or 6
                 self.dq = self._itodq(ip)
         else:
             # If string is in CIDR or netmask notation
@@ -195,10 +199,7 @@ class IP(object):
         >>> print ip.bin()
         01111111000000000000000000000001
         '''
-        h = hex(self.ip).lower().rstrip('l')
-        b = ''.join(self._bitmask[x] for x in h[2:])
-        l = self.v == 4 and 32 or 128
-        return ''.join('0' for x in xrange(len(b), l)) + b
+        return bin(self.ip).split('b')[1].rjust(self.mask, '0')
 
     def hex(self):
         '''
@@ -420,6 +421,12 @@ class IP(object):
         else:
             return self
 
+    def to_reverse(self):
+        if self.v == 4:
+            return '.'.join(list(self.dq.split('.')[::-1]) + ['in-addr', 'arpa'])
+        else:
+            return '.'.join(list(self.hex())[::-1] + ['ip6', 'arpa'])
+
     def to_tuple(self):
         '''
         Used for comparisons.
@@ -529,27 +536,44 @@ class Network(IP):
     def __eq__(self, other):
         return self.size() == IP(other).size()
 
+    def __getitem__(self, key):
+        if isinstance(key, slice):
+            # Work-around IPv6 subnets being huge. Slice indices don't like
+            # long int.
+            x = key.start or 0
+            slice_stop = (key.stop or self.size()) - 1
+            slice_step = key.step or 1
+            arr = list()
+            while x < slice_stop:
+                arr.append(IP(long(self)+x))
+                x += slice_step
+            return tuple(arr)
+        else:
+            return IP(long(self)+key)
+
     def __iter__(self):
         '''
         Generate a range of ip addresses within the network.
 
         >>> for ip in Network('192.168.114.0/30'):
         ...     print str(ip)
-        ... 
+        ...
         192.168.114.0
         192.168.114.1
         192.168.114.2
         192.168.114.3
         '''
-        for ip in (IP(long(self)+x) for x in xrange(0, self.size())):
-            yield ip
+        x = 0L
+        while x < self.size():
+            yield IP(long(self)+x)
+            x += 1L
 
     def has_key(self, ip):
         '''
         Check if the given ip is part of the network.
 
         :param ip: the ip address
-        :type ip: :class:`IP` or str or long or int 
+        :type ip: :class:`IP` or str or long or int
 
         >>> net = Network('192.0.2.0/24')
         >>> net.has_key('192.168.2.0')
@@ -598,6 +622,8 @@ if __name__ == '__main__':
             print 'network...:', net.network()
             print 'broadcast.:', net.broadcast()
         print 'first host:', net.host_first()
+        print 'reverse...:', net.host_first().to_reverse()
         print 'last host.:', net.host_last()
+        print 'reverse...:', net.host_last().to_reverse()
         for ip in test_ip:
             print '%s in network: ' % ip, ip in net
